#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['jsaxonpy']

package_data = \
{'': ['*']}

package_dir = \
{'': 'src'}

install_requires = \
['pyjnius >= 1.0.0, < 2.0.0']

extras_require = \
{'dev': ['tox>=3.3.0', 'bump2version'],
 'test': ['pytest >=7.2.0',
          'pytest-xdist',
          'pytest-cov',
          'pytest-html',
          'hypothesis',
          'black',
          'isort',
          'flake8']}

setup(name='jsaxonpy',
      version='0.4.3',
      description='Java Saxon XSLT transformations in your python applications',
      author=None,
      author_email='Andrey Kolotev <kolotev@gmail.com>',
      url=None,
      packages=packages,
      package_data=package_data,
      package_dir=package_dir,
      install_requires=install_requires,
      extras_require=extras_require,
      python_requires='>=3.7',
     )
